function KeystoreManager(url) {
    this._constructor(url);
}
(function() { // create local scope;

    KeystoreManager.Messages = {TITLE: "Generate Key", LOADING: "Loading...", GENERATING: "Generating..."};

    var IFRAME_ID = "GenerateKeyFormHolder";
    var BLANK_URL = "about:blank";

    KeystoreManager.prototype = {

        url: null,
        dialog: null,
        form: null,
        formLoader: null,
        messageBar: null,

        _constructor: function(url) {
            this.url = url;
        },

        setGlobalMessageBar: function(bar) {
            this.messageBar = bar;
        },

        init: function() {
            if (this.dialog == null) {
                this.dialog = new GenerateKeyDialog(this);
                this.dialog.setTitle(KeystoreManager.Messages.TITLE);
                this.formLoader = this._createFormLoader();
                Utils.setStyle(this.formLoader, {display: "none"});
                this.dialog.setContent(this.formLoader);
            }
        },

        onStart: function() {
            this.init();

            this.dialog.setButtonEnabled(ModalDialog.BUTTON_OK, false);
            this.dialog.setMessage(KeystoreManager.Messages.LOADING);
            this.dialog.pack();
            this.dialog.setVisible(true);

            this.formLoader.src = encodeURI(this.url);
        },

        onFinish: function(url, message) {
            if (this.dialog != null) {
                this.dialog.setVisible(false);
            }

            Utils.setStyle(this.formLoader, {widt: 0, height: 0, display: "block"});
            this.formLoader.src = encodeURI(url);

            this._setGlobalMessage(message);
        },

        onSubmit: function() {
            this.dialog.setMessage(KeystoreManager.Messages.GENERATING);
            Utils.setStyle(this.formLoader, {display: "none"});
            this.dialog.setButtonEnabled(ModalDialog.BUTTON_OK, false);
            this.dialog.pack();

            this.onFormSubmit(this.form);
            this.form = null;
        },

        onCancel: function() {
            this.dialog.setMessage(null);
            this.dialog.setVisible(false);
            this.form = null;
        },

        onFormLoaded: function(form) {
            this._setGlobalMessage(null);
            this.dialog.setMessage(null);
            Utils.setStyle(this.formLoader, {height: "100%", display: "block"});

            if (form != null) {
                this.form = form;
                var thisObj = this;
                form.onsubmit = function() {
                    return thisObj.onFormSubmit(this);
                };
                this.dialog.setButtonEnabled(ModalDialog.BUTTON_OK, true);
                //todo: check opera
                this.dialog.setSize(580, 320);
            }
        },

        onFormSubmit: function(form) {
            if (form == null) {
                return false;
            }

            form.submit();
            return true;
        },

        _setGlobalMessage: function(text) {
            while (this.messageBar.hasChildNodes()) {
                this.messageBar.removeChild(this.messageBar.lastChild);
            }

            if (text != null) {
                this.messageBar.appendChild(this._createSuccessMessage(text));
            }
        },

        _createSuccessMessage: function(text) {
            var message = document.createElement("H3");
            message.appendChild(document.createTextNode(text));

            var result = document.createElement(Tag.DIV);
            Utils.addCssClass(result, "alertbox");
            result.appendChild(message);

            return result;
        },

        _createFormLoader: function() {
            var result = document.getElementById(IFRAME_ID);

            if (result == null) {
                var holder = document.body.appendChild(document.createElement("div"));
                holder.innerHTML = "<iframe src='" + BLANK_URL + "' id='" + IFRAME_ID + "' name='" + IFRAME_ID + "' "
                        + "frameborder='0' style='width:100%;height:0;border:none;'></iframe>";
                holder.style.position = "relative";
                holder.style.top = 0;
                holder.style.left = 0;

                result = holder.getElementsByTagName("iframe")[0];
            }

            return result;
        }

    };
})();