var FlashUploader = Class.create(Uploader, {

    id: null,
    postData: null,
    flashComponent: null,
    eventQueue: null,
    eventHandlers: null,
    debug: null,

    _constructor: function(id) {
        this.id = id || (FlashUploader.Component.ID + new Date().getTime());
        this.debug = false;
        this.eventQueue = new Array();
        this.eventHandlers = new Array();
        this.eventHandlers["readyStateChanged"] = this._handleStateChanged.bind(this);
        this.eventHandlers["fileSelect"] = this._handleFileSelect.bind(this);
        this.eventHandlers["fileUpload"] = this._handleFileUpload.bind(this);
        this.eventHandlers["error"] = this._handleError.bind(this);

        var listener = {
            onFileSelect: function() {},
            onFileRemove: this._handleFileRemove.bind(this),
            onSubmit: this.onStart.bind(this),
            onCancel: this.onCancel.bind(this)
        };

        this.view = new UploadView(listener);
        this.postData = this._getCookieData();

        Utils.setEnabled(this.view.getOverflowElement(), false);

        try {
            this.flashComponent = this._createFlashComponent();
            this._position();
            FlashUploader.CallBacks[this.id] = this;

            // remove browser chooser
            this.dataChannel = null;
            this.chooser = null;
        } catch (e) {
            delete FlashUploader.CallBacks[this.id];
        }
    },

    toString: function() {
        return "Flash Uploader";
    },

    destroy: function() {
        this.cancel();
        this._destroyFlashComponent();

        this.eventQueue = null;
        this.postData = null;
        this.view = null;
        this.uploadForm = null;
        this.progressPresenter = null;
    },

    onStart: function() {
        if (this.view.getFileCount() == 0) {
            alert(Uploader.Messages.NO_FILES_SELECTED);
        } else {
            Utils.setStyle(this.flashComponent, "visibility:hidden");
            this.uploadForm.onStart();
        }

        return false;
    },

    onFinish: function() {
        this._restartTimeoutTimer();

        if (this.view.getFileCount() > 0 && this._enqueueFile()) {
            return;
        }

        this.timeoutTimer.stop();
        this.progressPresenter.finish();
        this._destroyFlashComponent();

        // todo: refactor
        window.refreshAction();
    },

    upload: function() {
        // todo: refactor
        window.stopInactivityService();

        this.progressPresenter.start();
        this.progressPresenter.setIndeterminate();

        this.timeoutTimer.start();
        this._invokeFlash(this.flashComponent, "upload", FlashUploader.URL, this._getCookieData());
    },

    cancel: function() {
        this.timeoutTimer.stop();
        this._invokeFlash(this.flashComponent, "cancel");
        this.progressPresenter.cancel();
    },

    _enqueueFile: function() {
        return this._invokeFlash(this.flashComponent, "enqueueFile");
    },

    _removeAllFiles: function(files) {
        return this._invokeFlash(this.flashComponent, "removeFile", files);
    },

    _handleStateChanged: function(state) {
        if (state == "complete") {
            Utils.setEnabled(this.view.getOverflowElement(), true);
            Utils.setStyle(this.flashComponent, {visibility: "visible"});
        } else if (state == "uninitialized") {
            throw new Error("Cannot initialize Flash upload component");
        }
    },

    _handleError: function(name, message) {
        if (this.debug) {
            if (window.console) {
                window.console.log("Error: " + name + ", " + message);
            } else {
                alert("Error: " + name + ", " + message);
            }
        }
    },

    _handleFileSelect: function(filenames) {
        for (var i = 0; i < filenames.length; ++i) {
            this.view.addFile(filenames[i]);
        }
    },

    _handleFileRemove: function(files) {
        if (this._removeAllFiles(files)) {
            for (var i = 0; i < files.length; ++i) {
                this.view.removeFile(files[i]);
            }
        }
    },

    _handleFileUpload: function(filename) {
        this.view.removeFile(filename);
    },

    _position: function() {
        if (!this.flashComponent) { return; }

        var lessor = this.view.getOverflowElement();
        var lessee = this.flashComponent.parentNode;

        var dimension = Utils.getDimension(lessor);
        var offset = document.viewport.getOffsetFor(lessor);
        var scrollOffset = document.viewport.getScrollOffsets();
        Utils.setStyle(lessee, {
            width: dimension.width + "px", height: dimension.height + "px",
            top: (offset.top + scrollOffset.top) + "px", left: (offset.left + scrollOffset.left) + "px"
        });
    },

    _invokeFlash: function(obj, method) {
        var args = $A(arguments).slice(2);
        var returnString = obj.CallFunction('<invoke name="' + method + '" returntype="javascript">'
                                            + __flash__argumentsToXML(args, 0) + '</invoke>');
        return eval(returnString);
    },

    _flashCallback: function() {
        var args = $A(arguments);
        var event = args.shift();
        var handler = this.eventHandlers[event];

        if (handler == null) {
            return;
        }

        this.eventQueue.push(handler);

        (function() {
            var fn = this.eventQueue ? this.eventQueue.shift() : null;
            if (Object.isFunction(fn)) {
                fn.apply(this, args);
            }
        }.bind(this).delay(0));
    },

    _getCookieData: function() {
        var result = {};

        var cookies = document.cookie.split(/;\s*/);
        for (var i = 0; i < cookies.length; ++i) {
            var pair = cookies[i].split("=");
            if (pair.length == 2) {
                result[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1]);
            }
        }
        return result;
    },

    _getFlashVariables: function() {
        return "fireCallback=FlashUploader.CallBacks." + encodeURIComponent(this.id) + "._flashCallback";
    },

    _createFlashComponent: function() {
        var swfUrl = FlashUploader.Component.URL;
        var html = "<object id='" + this.id + "' name='" + this.id + "' data='" + swfUrl + "' "
                   + "type='application/x-shockwave-flash' width='100%' height='100%'>"
                   + "<param name='movie' value='" + swfUrl + "'/>"
                   + "<param name='allowScriptAccess' value='always'/>"
                   + "<param name='wMode' value='transparent'/>"
                   + "<param name='quality' value='high'/>"
                   + "<param name='menu' value='false'/>"
                   + "<param name='flashvars' value='" + this._getFlashVariables() + "'/>";
        var holder = document.createElement("div");
        Utils.setStyle(holder, {
            width: 1, height: 1, position: "absolute",
            zIndex: FlashUploader.Component.Z_INDEX,
            overflow: "hidden",  visibility: "visible"
        });
        holder.innerHTML = html;
        document.body.appendChild(holder);

        Event.observe(window, "resize", this._position.bind(this));
        Event.observe(window, "load", this._position.bind(this));

        var flash = $(this.id);
        // http://kb2.adobe.com/cps/400/kb400730.html
        if (window[this.id] == undefined) {
            window[this.id] = flash;
        }

        return flash;
    },

    _destroyFlashComponent: function() {
        if (this.flashComponent == null) {
            return;
        }

        var component = this.flashComponent;
        var id = component.id;

        // DOM/JS IE 6/7 memory leak workaround
        if (typeof component.CallFunction === "unknown") {
            for (var property in component) {
                try {
                    if (Object.isFunction(component[property])) {
                        component[property] = null;
                    }
                } catch (ignored1) {}
            }

            try {
                document.body.removeChild(component.parentNode);
                // component.parentNode.removeChild(component);
            } catch (ignored2) {}
        }

        // http://kb2.adobe.com/cps/400/kb400730.html
        window[id] = null;
        // http://bugs.adobe.com/jira/browse/FP-529
        window["__flash__removeCallback"] = function (instance, name) {
            if (instance != null) {
                instance[name] = null;
            }
        };

        this.flashComponent = null;
    }

});

FlashUploader.available = false;
FlashUploader.Component = {ID: "FlashUploader", URL: "", Z_INDEX: 1000};
FlashUploader.RequiredVersion = {MAJOR: 9, MINOR: 0,  REVISION: 0};
FlashUploader.CallBacks = [];
FlashUploader.URL = "";

(function(){
    // Flash Player Version Detection in AC_RunActiveContent.js
    if (window.DetectFlashVer) {
        FlashUploader.available = DetectFlashVer(FlashUploader.RequiredVersion.MAJOR,
                                                 FlashUploader.RequiredVersion.MINOR,
                                                 FlashUploader.RequiredVersion.REVISION);
    }
})();