function ProgressBar(container) {
    this._constructor(container);
}
(function() { // create local scope;

    ProgressBar.Css = {DEFAULT: "progressbar", PROGRESS_TEXT: "progress", BOX_RECTANGLE: "bar"};

    var REPAINT_INTERVAL = 55;
    var MOVING_BOX_WIDTH = 5;
    var MOVING_STEP = 3;

    ProgressBar.prototype = {

        container: null,
        boxRectangle: null,
        progressText: null,
        progress: null,
        indeterminate: null,
        boxWidth: null,
        boxPosition: null,
        boxDirection: null,
        timer: null,

        _constructor: function(container) {
            this._compose(container);

            this.progress = 0;
            this.boxPosition = 0;
            this.boxWidth = MOVING_BOX_WIDTH;
            this.indeterminate = false;
            this.setVisible(false);
        },

        isVisible: function() {
            return Utils.isVisible(this.container);
        },

        setVisible: function(value) {
            Utils.setVisible(this.container, value);
        },

        isIndeterminate: function() {
            return this.indeterminate;
        },

        setIndeterminate: function(value) {
            this.indeterminate = value;

            if (value) {
                this._startAnimation();
            } else {
                this._stopAnimation();
            }
        },

        setProgress: function(value) {
            this.progress = Math.max(0, Math.min(100, value));
            if (this.indeterminate) {
                this.setIndeterminate(false);
            }
            this._repaint();
        },

        _compose: function(parent) {
            this.container = document.createElement(Tag.DIV);
            Utils.addCssClass(this.container, ProgressBar.Css.DEFAULT);

            this.progressText = document.createElement(Tag.SPAN);
            Utils.addCssClass(this.progressText, ProgressBar.Css.PROGRESS_TEXT);
            this.container.appendChild(this.progressText);

            this.boxRectangle = document.createElement(Tag.SPAN);
            Utils.addCssClass(this.boxRectangle, ProgressBar.Css.BOX_RECTANGLE);
            this.container.appendChild(this.boxRectangle);

            if (parent.firstChild) {
                parent.insertBefore(this.container, parent.firstChild);
            } else {
                parent.appendChild(this.container);
            }
        },

        _repaint: function() {
            if (!this.isVisible()) {
                return;
            }

            if (!this.indeterminate) {
                var percent = this.progress + "%";
                this.progressText.innerHTML = percent;
                Utils.setStyle(this.boxRectangle, {width: percent});
            } else {
                if ((this.boxPosition) >= (100 - this.boxWidth)) {
                    this.boxDirection = -1;
                } else if (this.boxPosition <= 0) {
                    this.boxDirection = 1;
                }

                this.boxPosition += (this.boxDirection * MOVING_STEP);
                var posPercent = Math.max(0, Math.min(100 - this.boxWidth, this.boxPosition)) + "%";
                Utils.setStyle(this.boxRectangle, {left: posPercent});
            }
        },

        _startAnimation: function() {
            Utils.setStyle(this.progressText, {visibility: "hidden"});
            Utils.setStyle(this.boxRectangle, {width: this.boxWidth + "%", position: "relative", left: this.boxPosition + "%"});

            this.timer = window.setInterval(function() {
                this._repaint();
            }.bind(this), REPAINT_INTERVAL);
        },

        _stopAnimation: function() {
            window.clearInterval(this.timer);
            this.timer = null;

            Utils.setStyle(this.progressText, {visibility: "visible"});
            Utils.setStyle(this.boxRectangle, "position:inherit;left:auto"); // bug in IE7
            this._repaint();
        }
    };

})();