function UploadFormDialog(listener) {
    this._constructor(listener);
}
(function() { // create local scope;

    UploadFormDialog.Buttons = {OK: 'OK', CANCEL: 'Cancel'};
    UploadFormDialog.Labels = {FORM: "Form", DESCRIPTION: "Description"};
    UploadFormDialog.Messages = {INFO: "Select form to use for file upload and click OK or click Cancel to upload files without form data."};

    var MIN_SIZE = {width: 400, height: 245};

    UploadFormDialog.prototype = {

        listener: null,
        dialog: null,
        formsBox: null,
        formDescElement: null,

        _constructor: function(listener) {
            this.listener = listener;
            this.dialog = new ModalDialog("UploadFormDialog");
            this.formsBox = document.createElement(Tag.SELECT);
            this.formsBox.name = "formname";
            this.formsBox.id = this.formsBox.name;
            this.formDescElement = document.createElement(Tag.SPAN);
            this.formDescElement.id = "formdescription";
            this.formDescriptions = new Array();

            var thisObj = this;
            this.formsBox.onchange = function() { thisObj._updateFormDescription(); };
            this.dialog.addButton(ModalDialog.BUTTON_OK, UploadFormDialog.Buttons.OK, function() { thisObj.onSubmit(); });
            this.dialog.addButton(ModalDialog.BUTTON_CANCEL, UploadFormDialog.Buttons.CANCEL, function() { thisObj.onCancel(); });
        },

        addForm: function(name, description) {
            var item = document.createElement(Tag.OPTION);
            item.appendChild(document.createTextNode(name));
            item.value = name;
            item.title = description;

            this.formsBox.appendChild(item);
        },

        getSelectedForm: function() {
            return $F(this.formsBox);
        },

        firstPage: function() {
            this.dialog.setContent(this._createFormSelectionPanel());
            var viewport = document.viewport.getDimensions();
            this.setSize(viewport.width * 0.4, viewport.height * 0.37);
        },

        setContent: function(content) {
            this.dialog.setContent(content);
        },

        setTitle: function(title) {
            this.dialog.setTitle(title);
        },

        setButtonEnabled: function(name, enable) {
            this.dialog.setButtonEnabled(name, enable);
        },

        isVisible: function() {
            return this.dialog.isVisible();
        },

        setVisible: function(visible) {
            this.dialog.setVisible(visible);
            this._registerKeyHandlers();
            this._updateFormDescription();
        },

        setSize: function(width, height) {
            this.dialog.setSize(Math.max(MIN_SIZE.width, width), Math.max(MIN_SIZE.height, height));
        },

        pack: function() {
            this.dialog.pack();
        },

        onSubmit: function() {
            this.listener.onSubmit();
        },

        onCancel: function() {
            this.listener.onCancel();
        },

        processKeyEvent: function(ke) {
            var code = ke.getKeyCode();

            switch (code) {
                case KeyEvent.VK_ENTER:
                    ke.cancel();
                    this.onSubmit();
                    break;

                case KeyEvent.VK_ESCAPE:
                    ke.cancel();
                    this.onCancel();
                    break;
            }
        },

        _createFormSelectionPanel: function() {
            var panel = document.createElement(Tag.DIV);

            var infoPanel = document.createElement(Tag.DIV);
            infoPanel.appendChild(document.createTextNode(UploadFormDialog.Messages.INFO));
            Utils.setStyle(infoPanel, "padding: 2em 1.5em 2em 4em;");
            panel.appendChild(infoPanel);

            var formPanel = document.createElement(Tag.DIV);
            formPanel.appendChild(Utils.createLabel(UploadFormDialog.Labels.FORM, this.formsBox.id));
            formPanel.appendChild(this.formsBox);
            formPanel.appendChild(document.createElement("BR"));
            formPanel.appendChild(Utils.createLabel(UploadFormDialog.Labels.DESCRIPTION, this.formDescElement.id));
            formPanel.appendChild(this.formDescElement);
            Utils.setStyle(formPanel, "padding-left: 5em; line-height: 2");
            panel.appendChild(formPanel);

            return panel;
        },

        _registerKeyHandlers: function() {
            var km = KeyboardManager.getInstance();
            if (this.isVisible()) {
                km.registerHandler(KeyEvent.VK_ENTER, this);
                km.registerHandler(KeyEvent.VK_ESCAPE, this);
            } else {
                km.unregisterHandler(KeyEvent.VK_ENTER, this);
                km.unregisterHandler(KeyEvent.VK_ESCAPE, this);
            }
        },

        _updateFormDescription: function() {
            var description = this._getFormDescripton($F(this.formsBox));
            this._setDescription(description || "");
        },

        _getFormDescripton: function(name) {
            for (var i = 0; i < this.formsBox.options.length; ++i) {
                var item = this.formsBox.options[i];
                if (name == item.value) {
                    return item.title;
                }
            }
            return null;
        },

        _setDescription: function(description) {
            var descElement = document.createTextNode(description);

            if (this.formDescElement.firstChild) {
                this.formDescElement.replaceChild(descElement, this.formDescElement.firstChild);
            } else {
                this.formDescElement.appendChild(descElement);
            }
        }

    };
})();