function Uploader() {
    this._constructor();
}
(function() {

    Uploader.Messages = {NO_FILES_SELECTED: "No files selected.", CONNECTION_TIMEOUT: "Connection timeout."};

    var CONNECTION_TIMEOUT = 60000;

    Uploader.prototype = {

        view: null,
        chooser: null,
        uploadForm: null,
        progressPresenter: null,
        dataChannel: null,
        timeoutTimer: null,

        _constructor: function() {
            var listener = {
                onFileSelect: this._handleFileSelect.bind(this),
                onFileRemove: this._handleFileRemove.bind(this),
                onSubmit: this.onStart.bind(this),
                onCancel: this.onCancel.bind(this)
            };

            this.view = new UploadView(listener);
            this.uploadForm = new UploadForm(this);
            this.chooser = new StandardFileChooser(listener);
            this.chooser.install(this.view.getForm());
            this.progressPresenter = new ProgressPresenter(listener);
            this.dataChannel = new IFrameChannel("UploadDataChannel");

            this.timeoutTimer = new Timer(this.onTimeout.bind(this), CONNECTION_TIMEOUT);
            this.timeoutTimer.setRepeats(false);
        },

        toString: function() {
            return "Uploader";
        },

        setConnectionTimeout: function(value) {
            this.timeoutTimer.setDelay(value || CONNECTION_TIMEOUT);
        },

        addUploadForm: function(name, description) {
            this.uploadForm.addForm(name, description);
        },

        getFilenames: function() {
            return this.view.getFilenames();
        },

        onStart: function() {
            if (this.view.getFileCount() == 0) {
                alert(Uploader.Messages.NO_FILES_SELECTED);
            } else {
                this.uploadForm.onStart();
            }

            return false;
        },

        onFinish: function() {
            this.timeoutTimer.stop();
            this.dataChannel.close();
            this.progressPresenter.finish();

            // todo: refactor
            window.refreshAction();
        },

        onCancel: function() {
            this.cancel();
            this.view.clear();

            // todo: refactor
            window.cancelUploadAction();
        },

        onTimeout: function() {
            this.cancel();
            alert(Uploader.Messages.CONNECTION_TIMEOUT);

            // todo: refactor
            window.refreshAction();
        },

        onUploadPending: function() {
            this._restartTimeoutTimer();
            this.progressPresenter.onUploadPending();
        },

        onUploadProgress: function(percent, status, time) {
            this._restartTimeoutTimer();
            this.progressPresenter.onUploadProgress(percent, status, time);
        },

        onCopyProgress: function(percent, status, time) {
            this._restartTimeoutTimer();
            this.progressPresenter.onCopyProgress(percent, status, time);
        },

        onUploadFormLoaded: function(form, requiredInputs) {
            this.uploadForm.setRequiredInputNames(form, requiredInputs);
            this.uploadForm.onFormLoaded();
        },

        onUploadFormComplete: function() {
            this.uploadForm.onFinish();
        },

        onUploadFormError: function(message) {
            this.uploadForm.onError(message);
        },

        upload: function() {
            // todo: refactor
            window.stopInactivityService();

            this.progressPresenter.start();
            this.progressPresenter.setIndeterminate();
            this.chooser.onBeforeSubmit();

            this.timeoutTimer.start();
            this.dataChannel.submit(this.view.getForm());
        },

        cancel: function() {
            this.timeoutTimer.stop();
            this.dataChannel.close();
            this.progressPresenter.cancel();
        },

        _handleFileSelect: function(filenames) {
            for (var i = 0; i < filenames.length; ++i) {
                this.view.addFile(filenames[i]);
            }
        },

        _handleFileRemove: function(filenames) {
            for (var i = 0; i < filenames.length; ++i) {
                var filename = filenames[i];
                this.view.removeFile(filename);
                this.chooser.remove(filename);
            }
        },

        _restartTimeoutTimer: function() {
            this.timeoutTimer.stop();
            this.timeoutTimer.start();
        }
    };
})();